import { CropGeometry, ResizeGeometry } from "upload-image-plugin/params/ImageGeometry";
import { LayerBlendMode } from "upload-image-plugin/params/CompositeBlendMode";
import { LayerGravityMode } from "upload-image-plugin/params/CompositeGravityMode";
import { BlurMode } from "upload-image-plugin/params/BlurMode";
import { SharpenMode } from "upload-image-plugin/params/SharpenMode";
import { RotateMode } from "upload-image-plugin/params/RotateMode";
import { ColorRGB } from "upload-image-plugin/params/ColorRGB";

export type ImagePipelineStep =
  | ResizeStep
  | CropStep
  | RotateStep
  | LayerStep
  | CompositeStep // Deprecated (use 'LayerStep' instead). Must include so that old transformations continue to be valid.
  | BlurStep
  | SharpenStep
  | TintStep
  | FlipStep
  | ModulateStep
  | GrayscaleStep
  | NegativeStep;

export type ImagePipelineStepType = ImagePipelineStep["type"];

export interface CropStep {
  /**
   * Geometry defining an image area and location.
   */
  geometry: CropGeometry;

  /**
   * Transformation Type
   */
  type: "crop";
}

export interface ResizeStep {
  /**
   * Geometry defining an image area.
   */
  geometry: ResizeGeometry;

  /**
   * Transformation Type
   */
  type: "resize";
}

export interface RotateStep {
  mode: RotateMode;

  /**
   * Transformation Type
   */
  type: "rotate";
}

export interface NegativeStep {
  /**
   * Negate Alpha Channel
   *
   * @description
   * Query Param: negate-alpha
   */
  negateAlpha: boolean;

  /**
   * Transformation Type
   *
   * @description
   * Query Param: negate
   * Query Param Value: true
   */
  type: "negative";
}

export interface TintStep {
  /**
   * Tint Color
   *
   * @description
   * Query Param: tint
   */
  color: ColorRGB;

  /**
   * Transformation Type
   */
  type: "tint";
}

export interface ModulateStep {
  /**
   * Brightness
   *
   * @description
   * Query Param: brightness
   *
   * @isInt
   * @minimum -100
   * @maximum 100
   */
  brightness: number;

  /**
   * Hue
   *
   * @description
   * Query Param: hue
   *
   * @isInt
   * @minimum -180
   * @maximum 180
   */
  hue: number;

  /**
   * Lightness
   *
   * @description
   * Query Param: lightness
   *
   * @isInt
   * @minimum -100
   * @maximum 100
   */
  lightness: number;

  /**
   * Saturation
   *
   * @description
   * Query Param: saturation
   *
   * @isInt
   * @minimum -100
   * @maximum 100
   */
  saturation: number;

  /**
   * Transformation Type
   */
  type: "modulate";
}

export interface GrayscaleStep {
  /**
   * Transformation Type
   *
   * @description
   * Query Param: grayscale
   * Query Param Value: true
   */
  type: "grayscale";
}

export interface BlurStep {
  mode: BlurMode;

  /**
   * Transformation Type
   */
  type: "blur";
}

export interface SharpenStep {
  mode: SharpenMode;

  /**
   * Transformation Type
   */
  type: "sharpen";
}

export interface FlipStep {
  /**
   * Axis
   *
   * @description
   * Query Param: flip
   */
  axis: "x" | "y";

  /**
   * Transformation Type
   */
  type: "flip";
}

export interface LayerStep {
  /**
   * Blend Mode
   *
   * @description
   * Query Param: layer-blend
   */
  blend?: LayerBlendMode;

  /**
   * Image Position (Relative)
   *
   * @description
   * Query Param: layer-gravity
   */
  gravity?: LayerGravityMode;

  /**
   * Image File ID (Legacy)
   */
  imageFileId?: string;

  /**
   * Image File Path
   *
   * @description
   * Query Param: layer-file-path
   */
  imageFilePath?: string;

  /**
   * Image Position (Absolute X)
   *
   * @description
   * Query Param: layer-x
   *
   * @isInt
   */
  left?: number;

  /**
   * Premultiply Base Image
   *
   * @description
   * Query Param: layer-premultiply
   */
  premultiplyBaseImage?: boolean;

  /**
   * Repeat Image
   *
   * @description
   * Query Param: layer-repeat
   */
  tile?: boolean;

  /**
   * Image Position (Absolute Y)
   *
   * @description
   * Query Param: layer-y
   *
   * @isInt
   */
  top?: number;

  /**
   * Transformation Type
   */
  type: "layer";
}

/**
 * Legacy. We renamed 'composite' to 'layer' and have kept this type here to highlight that the legacy data structures still exist.
 */
export interface CompositeStep extends LayerStep {
  type: "composite";
}
